'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by jcox on 6/3/16.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _q = require('q');

var Q = _interopRequireWildcard(_q);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CasperManager = function () {
    /**
     * Class to manage casper and DOM interactions
     * @param casper The casper object passed as global
     * @param phantom The phantom object passed as global
     */

    function CasperManager(casper, phantom) {
        _classCallCheck(this, CasperManager);

        this.casper = casper;
        this.phantom = phantom;

        this.failCount = 0;
    }

    /**
     * Set debug options in casper
     *
     * @param {Boolean} debug Whether to have debug mode turned on
     * @param {string} baseName The folder name for logging output and image captures
     *
     * @returns {void}
     */


    _createClass(CasperManager, [{
        key: 'setDebug',
        value: function setDebug(debug, baseName) {
            var self = this;

            if (this.casper.cli.get('trace') == true) {
                debug = true;
            }
            this.casper.on('page.error', function (msg, trace) {
                if (debug) {
                    self.casper.echo('Error: ' + msg, 'ERROR\n');
                    for (var i in trace) {
                        self.casper.echo(JSON.stringify(trace[i]));
                    }
                }
            });
            this.casper.on('remote.message', function (msg) {
                if (debug) {
                    self.casper.echo('remote console message: ' + msg);
                }
            });
            this.casper.test.on('fail', function () {
                self.failCount++;
                self.casper.capture('screenshots/' + baseName + '/' + baseName + '_fail_' + self.failCount + '.png');
                self.casper.echo("Failed - image captured");
            });
        }

        /**
         * Clicks an element in the DOM after waiting for the element to be there
         *
         * @param {string} selector The selector of the element to be clicked
         *
         * @returns {void}
         */

    }, {
        key: 'click',
        value: function click(selector) {
            var self = this;

            this.waitForSelector(selector, function () {
                self.casper.click(selector);
            });
        }

        /**
         * Retrieves the text from an element
         *
         * @param {string} selector The selector of the element from which to get the text
         *
         * @returns {*} Promise that resolves with the text
         */

    }, {
        key: 'getText',
        value: function getText(selector) {
            var self = this,
                deferred = Q.defer();

            this.waitForSelector(selector, function () {
                deferred.resolve(self.casper.fetchText(selector));
            });

            return deferred;
        }

        /**
         * Waits for a selector to be available on the page
         *
         * @param {string} selector The selector for which to wait
         * @param {Function} cb A callback function once the element is available
         *
         * @returns {void}
         */

    }, {
        key: 'waitForSelector',
        value: function waitForSelector(selector, cb) {
            var self = this;

            this.casper.then(function () {
                self.casper.waitForSelector(selector, cb);
            });
        }

        /**
         * Waits until selector no longer exists in the DOM
         *
         * @param {string} selector The selector of the element to wait for
         */

    }, {
        key: 'waitWhileSelector',
        value: function waitWhileSelector(selector) {
            var self = this;

            this.waitForSelector(selector, function () {
                self.casper.waitWhileSelector(selector);
            });
        }

        /**
         * Evaluates an expression in the current page DOM context
         *
         * @param {Object} params The params to be used in the evaluate callback
         * @param {Function} cb The script to execute against the DOM
         * @param {string} waitForSelector Optional selector to wait for before executing the callback
         *
         * @returns {*|promise}
         */

    }, {
        key: 'evaluate',
        value: function evaluate(params, cb, waitForSelector) {
            var self = this,
                deferred = Q.defer();

            if (waitForSelector) {
                this.waitForSelector(waitForSelector, function () {
                    var response = self.casper.evaluate(cb, params);
                    deferred.resolve(response);
                });
            } else {
                this.casper.then(function () {
                    var response = self.casper.evaluate(cb, params);
                    deferred.resolve(response);
                });
            }

            return deferred.promise;
        }

        /**
         * Types text into an input field
         *
         * @param {string} selector The selector of the field to type into
         * @param {string} keys The text to type
         */

    }, {
        key: 'sendKeys',
        value: function sendKeys(selector, keys) {
            var self = this;

            this.waitForSelector(selector, function () {
                self.casper.sendKeys(selector, keys);
            });
        }

        /**
         * Helper method for casper element getters
         * 
         * @param {string} selector The element selector
         * @param {Function} getter the casper method to call
         * @param {string?} attr (Optional) The attribute to get from the element
         * @returns {*|promise}
         */

    }, {
        key: 'elementGetHelper',
        value: function elementGetHelper(selector, getter, attr) {
            var deferred = Q.defer();

            this.waitForSelector(selector, function () {
                var arr = void 0;

                if (attr) {
                    arr = getter(selector, attr);
                } else {
                    arr = getter(selector);
                }

                deferred.resolve(arr.length == 1 ? arr[0] : arr);
            });

            return deferred.promise;
        }

        /**
         * Gets an attibute from all elements matching the selector
         *
         * @param {string} selector The element selector
         * @param {string} attr The attribute to get
         * 
         * @returns {*|promise}
         */

    }, {
        key: 'getAttribute',
        value: function getAttribute(selector, attr) {
            return this.elementGetHelper(selector, this.casper.getElementsAttribute, attr);
        }

        /**
         * Retrieves the boundaries (coordinates) of the elements with the given selector
         *
         * @param {string} selector The element selector
         * 
         * @returns {*|promise}
         */

    }, {
        key: 'getBounds',
        value: function getBounds(selector) {
            return this.elementGetHelper(selector, this.casper.getElementsBounds);
        }

        /**
         * Retrieves all of the info of the elements with the given selector
         *
         * @param {string} selector The element selector
         *
         * @returns {*|promise}
         */

    }, {
        key: 'getInfo',
        value: function getInfo(selector) {
            return this.elementGetHelper(selector, this.casper.getElementsInfo);
        }

        /**
         * Retrieves the casper instance
         * 
         * @returns {*}
         */

    }, {
        key: 'getCasper',
        value: function getCasper() {
            return this.casper;
        }

        /**
         * Retrieves the phantom instance
         * 
         * @returns {*}
         */

    }, {
        key: 'getPhantom',
        value: function getPhantom() {
            return this.phantom;
        }
    }]);

    return CasperManager;
}();

exports.default = CasperManager;